pub enum WindowSizePolicy {
Content,
User,
}Expand description
Defines how a windows size should be determined
Variants
Content
Use the content of the window to determine the size.
If you use this option, your root widget will be passed infinite constraints; you are responsible for ensuring that your content picks an appropriate size.
User
Use the provided window size