[−][src]Struct druid::PaintCtx
A context passed to paint methods of widgets.
Widgets paint their appearance by calling methods on the
render_ctx, which PaintCtx derefs to for convenience.
This struct is expected to grow, for example to include the
"damage region" indicating that only a subset of the entire
widget hierarchy needs repainting.
Fields
render_ctx: &'a mut Piet<'b>The render context for actually painting.
window_id: WindowIdMethods
impl<'a, 'b: 'a> PaintCtx<'a, 'b>[src]
pub fn is_hot(&self) -> bool[src]
Query the "hot" state of the widget.
See EventCtx::is_hot for
additional information.
pub fn is_active(&self) -> bool[src]
Query the "active" state of the widget.
See EventCtx::is_active for
additional information.
pub fn size(&self) -> Size[src]
Returns the layout size of the current widget.
See EventCtx::size for
additional information.
pub fn has_focus(&self) -> bool[src]
Query the focus state of the widget.
See EventCtx::has_focus for
additional information.
pub fn region(&self) -> &Region[src]
Returns the currently visible Region.
pub fn with_child_ctx(
&mut self,
region: impl Into<Region>,
f: impl FnOnce(&mut PaintCtx)
)[src]
&mut self,
region: impl Into<Region>,
f: impl FnOnce(&mut PaintCtx)
)
Creates a temporary PaintCtx with a new visible region, and calls
the provided function with that PaintCtx.
This is used by containers to ensure that their children have the correct visible region given their layout.