pub struct RoundedRectRadii {
pub top_left: f64,
pub top_right: f64,
pub bottom_right: f64,
pub bottom_left: f64,
}Expand description
Radii for each corner of a rounded rectangle.
The use of top as in top_left assumes a y-down coordinate space. Piet
(and Druid by extension) uses a y-down coordinate space, but Kurbo also
supports a y-up coordinate space, in which case top_left would actually
refer to the bottom-left corner, and vice versa. Top may not always
actually be the top, but top corners will always have a smaller y-value
than bottom corners.
Fields
top_left: f64The radius of the top-left corner.
top_right: f64The radius of the top-right corner.
bottom_right: f64The radius of the bottom-right corner.
bottom_left: f64The radius of the bottom-left corner.
Implementations
Sourceimpl RoundedRectRadii
impl RoundedRectRadii
Sourcepub const fn new(
top_left: f64,
top_right: f64,
bottom_right: f64,
bottom_left: f64,
) -> RoundedRectRadii
pub const fn new( top_left: f64, top_right: f64, bottom_right: f64, bottom_left: f64, ) -> RoundedRectRadii
Create a new RoundedRectRadii. This function takes radius values for the four corners. The argument order is “top_left, top_right, bottom_right, bottom_left”, or clockwise starting from top_left.
Sourcepub const fn from_single_radius(radius: f64) -> RoundedRectRadii
pub const fn from_single_radius(radius: f64) -> RoundedRectRadii
Create a new RoundedRectRadii from a single radius. The radius
argument will be set as the radius for all four corners.
Sourcepub fn abs(&self) -> RoundedRectRadii
pub fn abs(&self) -> RoundedRectRadii
Takes the absolute value of all corner radii.
Sourcepub fn clamp(&self, max: f64) -> RoundedRectRadii
pub fn clamp(&self, max: f64) -> RoundedRectRadii
For each corner, takes the min of that corner’s radius and max.
Sourcepub fn as_single_radius(&self) -> Option<f64>
pub fn as_single_radius(&self) -> Option<f64>
If all radii are equal, returns the value of the radii. Otherwise,
returns None.