RawLabel

Struct RawLabel 

pub struct RawLabel<T> { /* private fields */ }
A widget that displays text data.

This requires the Data to implement TextStorage; to handle static, dynamic, or localized text, use Label.

Implementations

Source

impl<T: TextStorage> RawLabel<T>

pub fn new() -> Self

Create a new RawLabel.

Examples found in repository?
examples/markdown_preview.rs (line 117)
115fn build_root_widget() -> impl Widget<AppState> {
116    let label = Scroll::new(
117        RawLabel::new()
118            .with_text_color(Color::BLACK)
119            .with_line_break_mode(LineBreaking::WordWrap)
120            .lens(AppState::rendered)
121            .expand_width()
122            .padding((SPACER_SIZE * 4.0, SPACER_SIZE)),
123    )
124    .vertical()
125    .background(Color::grey8(222))
126    .expand();
127
128    let textbox = TextBox::multiline()
129        .lens(AppState::raw)
130        .controller(RichTextRebuilder)
131        .expand()
132        .padding(5.0);
133
134    Split::columns(label, textbox)
135}
More examples
Hide additional examples
examples/text.rs (line 113)
111fn build_root_widget() -> impl Widget<AppState> {
112    let label = Scroll::new(
113        RawLabel::new()
114            .with_text_color(Color::BLACK)
115            .controller(LabelController)
116            .background(Color::WHITE)
117            .expand_width()
118            .padding((SPACER_SIZE * 4.0, SPACER_SIZE))
119            .background(Color::grey8(222)),
120    )
121    .vertical();
122
123    let line_break_chooser = Flex::column()
124        .with_child(Label::new("Line break mode"))
125        .with_spacer(SPACER_SIZE)
126        .with_child(RadioGroup::column(vec![
127            ("Clip", LineBreaking::Clip),
128            ("Wrap", LineBreaking::WordWrap),
129            ("Overflow", LineBreaking::Overflow),
130        ]))
131        .lens(AppState::line_break_mode);
132
133    let alignment_picker = Flex::column()
134        .with_child(Label::new("Justification"))
135        .with_spacer(SPACER_SIZE)
136        .with_child(RadioGroup::column(vec![
137            ("Start", TextAlignment::Start),
138            ("End", TextAlignment::End),
139            ("Center", TextAlignment::Center),
140            ("Justified", TextAlignment::Justified),
141        ]))
142        .lens(AppState::alignment);
143
144    let controls = Flex::row()
145        .cross_axis_alignment(druid::widget::CrossAxisAlignment::Start)
146        .with_child(alignment_picker)
147        .with_spacer(SPACER_SIZE)
148        .with_child(line_break_chooser)
149        .padding(SPACER_SIZE);
150
151    Flex::column()
152        .cross_axis_alignment(druid::widget::CrossAxisAlignment::Start)
153        .with_child(controls)
154        .with_flex_child(label, 1.0)
155}
pub fn with_text_color(self, color: impl Into<KeyOrValue<Color>>) -> Self

Builder-style method for setting the text color.

The argument can be either a Color or a Key<Color>.

Examples found in repository?
examples/markdown_preview.rs (line 118)
115fn build_root_widget() -> impl Widget<AppState> {
116    let label = Scroll::new(
117        RawLabel::new()
118            .with_text_color(Color::BLACK)
119            .with_line_break_mode(LineBreaking::WordWrap)
120            .lens(AppState::rendered)
121            .expand_width()
122            .padding((SPACER_SIZE * 4.0, SPACER_SIZE)),
123    )
124    .vertical()
125    .background(Color::grey8(222))
126    .expand();
127
128    let textbox = TextBox::multiline()
129        .lens(AppState::raw)
130        .controller(RichTextRebuilder)
131        .expand()
132        .padding(5.0);
133
134    Split::columns(label, textbox)
135}
More examples
Hide additional examples
examples/text.rs (line 114)
111fn build_root_widget() -> impl Widget<AppState> {
112    let label = Scroll::new(
113        RawLabel::new()
114            .with_text_color(Color::BLACK)
115            .controller(LabelController)
116            .background(Color::WHITE)
117            .expand_width()
118            .padding((SPACER_SIZE * 4.0, SPACER_SIZE))
119            .background(Color::grey8(222)),
120    )
121    .vertical();
122
123    let line_break_chooser = Flex::column()
124        .with_child(Label::new("Line break mode"))
125        .with_spacer(SPACER_SIZE)
126        .with_child(RadioGroup::column(vec![
127            ("Clip", LineBreaking::Clip),
128            ("Wrap", LineBreaking::WordWrap),
129            ("Overflow", LineBreaking::Overflow),
130        ]))
131        .lens(AppState::line_break_mode);
132
133    let alignment_picker = Flex::column()
134        .with_child(Label::new("Justification"))
135        .with_spacer(SPACER_SIZE)
136        .with_child(RadioGroup::column(vec![
137            ("Start", TextAlignment::Start),
138            ("End", TextAlignment::End),
139            ("Center", TextAlignment::Center),
140            ("Justified", TextAlignment::Justified),
141        ]))
142        .lens(AppState::alignment);
143
144    let controls = Flex::row()
145        .cross_axis_alignment(druid::widget::CrossAxisAlignment::Start)
146        .with_child(alignment_picker)
147        .with_spacer(SPACER_SIZE)
148        .with_child(line_break_chooser)
149        .padding(SPACER_SIZE);
150
151    Flex::column()
152        .cross_axis_alignment(druid::widget::CrossAxisAlignment::Start)
153        .with_child(controls)
154        .with_flex_child(label, 1.0)
155}
pub fn with_text_size(self, size: impl Into<KeyOrValue<f64>>) -> Self

Builder-style method for setting the text size.

The argument can be either an f64 or a Key<f64>.

pub fn with_font(self, font: impl Into<KeyOrValue<FontDescriptor>>) -> Self

Builder-style method for setting the font.

The argument can be a FontDescriptor or a Key<FontDescriptor> that refers to a font defined in the Env.

pub fn with_line_break_mode(self, mode: LineBreaking) -> Self

Builder-style method to set the LineBreaking behaviour.

Examples found in repository?
examples/markdown_preview.rs (line 119)
115fn build_root_widget() -> impl Widget<AppState> {
116    let label = Scroll::new(
117        RawLabel::new()
118            .with_text_color(Color::BLACK)
119            .with_line_break_mode(LineBreaking::WordWrap)
120            .lens(AppState::rendered)
121            .expand_width()
122            .padding((SPACER_SIZE * 4.0, SPACER_SIZE)),
123    )
124    .vertical()
125    .background(Color::grey8(222))
126    .expand();
127
128    let textbox = TextBox::multiline()
129        .lens(AppState::raw)
130        .controller(RichTextRebuilder)
131        .expand()
132        .padding(5.0);
133
134    Split::columns(label, textbox)
135}
pub fn with_text_alignment(self, alignment: TextAlignment) -> Self

Builder-style method to set the TextAlignment.

pub fn set_text_color(&mut self, color: impl Into<KeyOrValue<Color>>)

Set the text color.

The argument can be either a Color or a Key<Color>.

If you change this property, you are responsible for calling request_layout to ensure the label is updated.

pub fn set_text_size(&mut self, size: impl Into<KeyOrValue<f64>>)

Set the text size.

The argument can be either an f64 or a Key<f64>.

If you change this property, you are responsible for calling request_layout to ensure the label is updated.

pub fn set_font(&mut self, font: impl Into<KeyOrValue<FontDescriptor>>)

Set the font.

The argument can be a FontDescriptor or a Key<FontDescriptor> that refers to a font defined in the Env.

If you change this property, you are responsible for calling request_layout to ensure the label is updated.

pub fn set_line_break_mode(&mut self, mode: LineBreaking)

Set the LineBreaking behaviour.

If you change this property, you are responsible for calling request_layout to ensure the label is updated.

Examples found in repository?
examples/text.rs (line 72)
63    fn update(
64        &mut self,
65        child: &mut RawLabel<AppState>,
66        ctx: &mut UpdateCtx,
67        old_data: &AppState,
68        data: &AppState,
69        env: &Env,
70    ) {
71        if old_data.line_break_mode != data.line_break_mode {
72            child.set_line_break_mode(data.line_break_mode);
73            ctx.request_layout();
74        }
75        if old_data.alignment != data.alignment {
76            child.set_text_alignment(data.alignment);
77            ctx.request_layout();
78        }
79        child.update(ctx, old_data, data, env);
80    }
pub fn set_text_alignment(&mut self, alignment: TextAlignment)

Set the TextAlignment for this layout.

Examples found in repository?
examples/text.rs (line 76)
63    fn update(
64        &mut self,
65        child: &mut RawLabel<AppState>,
66        ctx: &mut UpdateCtx,
67        old_data: &AppState,
68        data: &AppState,
69        env: &Env,
70    ) {
71        if old_data.line_break_mode != data.line_break_mode {
72            child.set_line_break_mode(data.line_break_mode);
73            ctx.request_layout();
74        }
75        if old_data.alignment != data.alignment {
76            child.set_text_alignment(data.alignment);
77            ctx.request_layout();
78        }
79        child.update(ctx, old_data, data, env);
80    }
pub fn draw_at(&self, ctx: &mut PaintCtx<'_, '_, '_>, origin: impl Into<Point>)

Draw this label’s text at the provided Point, without internal padding.

This is a convenience for widgets that want to use Label as a way of managing a dynamic or localized string, but want finer control over where the text is drawn.

pub fn baseline_offset(&self) -> f64

Return the offset of the first baseline relative to the bottom of the widget.