pub struct TabsState<TP: TabsPolicy> { /* private fields */ }Expand description
This is the current state of the tabs widget as a whole. This expands the input data to include a policy that determines how tabs are derived, and the index of the currently selected tab
Implementations
Sourceimpl<TP: TabsPolicy> TabsState<TP>
impl<TP: TabsPolicy> TabsState<TP>