ValueTextBox

Struct ValueTextBox 

pub struct ValueTextBox<T> { /* private fields */ }
A TextBox that uses a Formatter to handle formatting and validation of its data.

There are a number of ways to customize the behaviour of the text box in relation to the provided Formatter:

Implementations

Source

impl<T: Data> ValueTextBox<T>

pub fn new( child: TextBox<String>, formatter: impl Formatter<T> + 'static, ) -> Self

Create a new ValueTextBox from a normal TextBox and a Formatter.

pub fn delegate(self, delegate: impl ValidationDelegate + 'static) -> Self

Builder-style method to set an optional ValidationDelegate on this textbox.

pub fn validate_while_editing(self, validate: bool) -> Self

Builder-style method to set whether or not this text box validates its contents during editing.

If true (the default) edits that fail validation (Formatter::validate_partial_input) will be rejected. If false, those edits will be accepted, and the text box will be updated.

pub fn update_data_while_editing(self, flag: bool) -> Self

Builder-style method to set whether or not this text box updates the incoming data during editing.

If false (the default) the data is only updated when editing completes.